"""
Copyright (c) 2015-2020 BurstIQ Inc - All Rights Reserved
Unauthorized copying of this file, via any medium is strictly prohibited.
Proprietary and confidential
"""

import getpass
import os
from enum import Enum, IntEnum
from typing import Union

import yaml


class QueryLang(IntEnum):
    TQL = 0
    FLOW = 1


class Target(IntEnum):
    BURSTCHAIN = 0
    IQ = 1


class AssetType(Enum):
    ASSET = 0
    SMART_CONTRACT = 1


class CliqProperties(object):
    def __init__(self, props_file: str):
        if os.path.isdir(props_file):
            print(f'Error: properties file path {props_file} is a directory, must be a file path')
            exit(0)

        # load from file, check required, add defaults
        try:
            with open(props_file, 'rt') as file:
                self._props = yaml.safe_load(file)
        except IOError as err:
            print(f'Unable to load properties file, {err}')

            gen = input('Would you like to generate one? (y/n): ')
            if gen == 'y':
                self._generate_prop_file(props_file)
            else:
                exit(0)

    def _generate_prop_file(self, props_file: str):
        try:
            print('Generating a basic Properties file.')
            username = None
            password = None
            client = None
            target = None

            while not username:
                username = input('Please provide a username: ')

            while not password:
                password = getpass.getpass('Please provide the associated password: ')

            url = input('Please provide the host? (default is https://testnet.burstiq.com): ')
            # default to testnet
            if url is None or url == '':
                url = 'https://testnet.burstiq.com'

            while not client:
                client = input('Please provide a client name: ')

            while not target or not (target.upper() == 'IQ' or target.upper() == 'BURSTCHAIN'):
                target = input('What is the target for your queries? (IQ or BURSTCHAIN): ')

            input_path = input('What is your input file path (default is ~/)')
            output_path = input('What is your output file path (default is ~/)')

            default = {
                'url': url,
                'client': client,
                'target': target,
                'input_path': input_path,
                'output_path': output_path
            }
            user_info = {
                'username': username,
                'password': password
            }
            profiles = {
                'default': default
            }
            yam = {
                'userinfo': user_info,
                'profiles': profiles
            }

            with open(props_file, 'w') as outfile:
                yaml.dump(yam, outfile, default_flow_style=False)

            print(f'CLIQ properties file generated, It is saved to {props_file}, '
                  'you can make changes to the info there and add more profiles.')

            with open(props_file, 'rt') as file:
                self._props = yaml.safe_load(file)

        except Exception as err:
            print(f'Unable to Generate a cliq_properties.yaml file, {err}')

    def get_profiles(self) -> None:
        """
        Prints the name of each profile defined in the properties file.
        """
        profiles = self._props.get('profiles')
        print('Profiles: ')
        for profile in profiles:
            print(profile)
        print()

    def get_profile(self, profile_name: str) -> dict:
        """
        Returns a profiles values by looking it up using the name.
        """
        profiles = self._props.get('profiles')
        return profiles.get(profile_name)

    def get_user_info(self) -> dict:
        """
        Returns the User Info.
        """
        return self._props.get('userinfo')

    def get_query_lang(self) -> Union[None, QueryLang]:
        """
        Returns the Query Language defined in the Properties file.
        """
        return QueryLang[self._props.get('query_lang').upper()]

    def get_file_path(self) -> str:
        """
        Returns the file path defined in the Properties file.
        """
        fp = self._props.get('file_path')
        if fp is None or fp == '':
            fp = './'

        return fp
