"""
Copyright (c) 2015-2020 BurstIQ Inc - All Rights Reserved
Unauthorized copying of this file, via any medium is strictly prohibited.
Proprietary and confidential
"""

from cmd import Cmd

from cliq_runner.cliq_properties import Target
from cliq_runner.interpreters.dict_interpreter import DictInterpreter
from cliq_runner.interpreters.generate_interpreter import GenerateInterpreter
from cliq_runner.interpreters.profile_interpreter import ProfileInterpreter
from cliq_runner.interpreters.set_interpreter import SetInterpreter
from cliq_runner.parsers.query_parser import QueryParser
from cliq_runner.runner_manager import RunnerManager
from cliq_runner.runners.dictionary_info import DictionaryInfo
from cliq_runner.runners.query import Query


class CliqInterpreter(Cmd):
    """
    This is just the cmd line interpreter
    All state is managed in QueryRunner
    """

    def __init__(self,
                 runner: RunnerManager,
                 dict_info: DictionaryInfo,
                 generate_interpreter: GenerateInterpreter,
                 dict_interpreter: DictInterpreter,
                 set_interpreter: SetInterpreter,
                 profile_interpreter: ProfileInterpreter):
        super().__init__()

        # manages all state necessary for running queries
        self._runner_manager = runner
        self._dictionary_info = dict_info
        self._generate_interpreter = generate_interpreter
        self._dict_interpreter = dict_interpreter
        self._set_interpreter = set_interpreter
        self._profile_interpreter = profile_interpreter

        Cmd.prompt = f'cliq> '

    def do_help(self, line: str):
        """
        Returns information about the Commands.
        """
        Cmd.doc_header = 'Documented commands (type "help <topic>"):'
        super(CliqInterpreter, self).do_help(line)
        self._dict_interpreter.do_help(line)
        self._set_interpreter.do_help(line)
        self._generate_interpreter.do_help(line)
        self._profile_interpreter.do_help(line)

    def do_info(self, line: str = None) -> None:
        """
        Returns information on the current configuration
        """
        try:
            self._runner_manager.print_info()
        except SystemExit as e:
            print('try again...')

    # noinspection PyMethodMayBeStatic
    # this must be a member function for the interpreter
    def do_exit(self, line: str) -> bool:
        """
        Will Exit CLIQ
        """
        print('Good Bye')
        return True

    def do_set(self, line: str) -> None:
        """
        This will set user info based on the sub command and value provided.

        Example:
            set target <IQ/BURSTCHAIN>
            set username <username>
            set password <password>
            set client <client_name>
            set private_id <private_id>
            set host_url <host_url>
        """
        try:
            self._set_interpreter.onecmd(line)
        except SystemExit as e:
            print('try again...')

    def do_generate(self, line: str) -> None:
        """
        Generate This will generate A Public or Private Id
        """
        try:
            self._generate_interpreter.onecmd(line)
        except SystemExit as e:
            print('try again...')

    def do_profile(self, line: str) -> None:
        """
        For running profile commands
        """
        try:
            self._profile_interpreter.onecmd(line)
        except SystemExit as e:
            print('try again...')

    def do_dict(self, line: str) -> None:
        """
        Dictionary related commands
        """
        try:
            self._dict_interpreter.onecmd(line)
        except SystemExit as e:
            print('try again...')

    def do_tql(self, line: str):
        """
        Run a TQL Query
        """
        try:
            args = QueryParser.get_query_values(line)
            Query.run_tql(self._runner_manager, args)
        except SystemExit as e:
            print('try again...')

    def do_flow(self, line: str):
        """
        Run a Flow Query
        """
        try:
            args = QueryParser.get_query_values(line)
            Query.run_flow(self._runner_manager, args)
        except SystemExit as e:
            print('try again...')

    def do_map_reduce(self, line: str):
        """
        Run a Map Reduce Query
        """
        if self._runner_manager.get_info().get_target() == Target.BURSTCHAIN:
            print(f'Map/Reduce not supported on burstchain at this time')
            return

        try:
            args = QueryParser.get_query_values(line)
            Query.run_map_reduce(self._runner_manager, args)
        except SystemExit as e:
            print('try again...')
