#!/usr/bin/env python3

"""
Copyright (c) 2015-2020 BurstIQ Inc - All Rights Reserved
Unauthorized copying of this file, via any medium is strictly prohibited.
Proprietary and confidential
"""

import argparse
import sys
from os.path import expanduser
from typing import List

from cliq_runner.parsers.query_parser import QueryParser

from cliq_runner.cliq_properties import CliqProperties
from cliq_runner.interpreters.cliq_interpreter import CliqInterpreter
from cliq_runner.interpreters.dict_interpreter import DictInterpreter
from cliq_runner.interpreters.generate_interpreter import GenerateInterpreter
from cliq_runner.interpreters.profile_interpreter import ProfileInterpreter
from cliq_runner.interpreters.set_interpreter import SetInterpreter
from cliq_runner.runner_manager import RunnerManager
from cliq_runner.runners.dictionary_info import DictionaryInfo


def _parse_args() -> (argparse.ArgumentParser,
                      argparse.Namespace):
    home = expanduser('~')

    # define cmd line args
    args = argparse.ArgumentParser(prog='cliq_runner')
    args = QueryParser.query_parser(args)
    args.add_argument('-po', '--props',
                      dest='props_file',
                      help='filename of properties file; defaults to ~/cliq_properties.yaml',
                      default=f'{home}/cliq_properties.yaml')

    return args


def _setup_cliq(props_file: str, profile_name: str) -> CliqInterpreter:
    # parse the properties file
    props = CliqProperties(props_file)

    # create query runner
    runner = RunnerManager(props, profile_name)

    dict_info = DictionaryInfo()

    # run cliq_runner
    cliq_dict = DictInterpreter(runner, dict_info)
    cliq_gen = GenerateInterpreter(runner)
    cliq_set = SetInterpreter(runner)
    cliq_profile = ProfileInterpreter(runner)

    # run cliq_runner
    return CliqInterpreter(runner, dict_info, cliq_gen, cliq_dict, cliq_set, cliq_profile)


def _go_interactive(args: List) -> bool:
    return args is None or args == '' or args == [] or args[0].startswith('-')


def main():
    # noinspection PyBroadException
    try:
        print('\n'
              + '--------------------------------------------------------------\n'
              + 'CLIQ (Command Line IQ)\n'
              + 'Copyright (c) 2015-2020 BurstIQ Inc\n'
              + '--------------------------------------------------------------\n')

        og_args = sys.argv[1:]

        if _go_interactive(og_args):
            # parse cmd line args

            args = _parse_args()
            opts = args.parse_args()
            cliq = _setup_cliq(opts.props_file, opts.profile)
            cliq.cmdloop()
        else:
            # parse the properties file
            og_args = sys.argv[2:]
            cmd = sys.argv[1:]
            args = _parse_args()
            opts = args.parse_args(og_args)

            home = expanduser('~')
            filename = f'{home}/cliq_properties.yaml'
            if opts.profile is None:
                profile = 'default'
            else:
                profile = opts.profile

            cliq = _setup_cliq(filename, profile)
            cliq.onecmd(' '.join(cmd))
    except BaseException as e:
        print(e)
        print()
        print('Good Bye')
        exit(0)


if __name__ == '__main__':
    main()
