"""
Copyright (c) 2015-2020 BurstIQ Inc - All Rights Reserved
Unauthorized copying of this file, via any medium is strictly prohibited.
Proprietary and confidential
"""
import json
import os
from argparse import Namespace
from os.path import expanduser
from typing import Dict

import requests
from requests.auth import HTTPBasicAuth

from cliq_runner.api_runner import ApiRunner
from cliq_runner.cliq_properties import Target
from cliq_runner.info.info import Info
from cliq_runner.runner_manager import RunnerManager


class Query:
    @staticmethod
    def run_map_reduce(runner_manager: RunnerManager,
                       args: Namespace) -> None:
        print('Running map reduce')
        api_path = f'{runner_manager.get_info().get_url(args)}/api/iq/' \
                   f'{runner_manager.get_info().get_client(args)}/mapreduce/query'

        auth = Query._get_auth(runner_manager.get_info(), args, runner_manager)

        # Read a Json file to assign each map reduce value
        if args.map_reduce is not None or args.map_reduce != '':
            try:
                with open(Query._standardize_file(runner_manager.get_info(), args.map_reduce)) as f:
                    body = json.load(f)

                resp = ApiRunner.run_post(api_path, body, auth[1])

                Query._do_output(runner_manager, args, resp)
            except Exception as err:
                print(f'Unable to Load Map Reduce File, {err}')
        else:
            print('You must provide a map reduce file using the flag --map_reduce <file_name>')

    @staticmethod
    def run_tql(runner_manager: RunnerManager,
                args: Namespace) -> None:
        """
        Runs a Query using the API Path, Auth and Query body.
        """
        try:
            query_body = Query._get_tql_request_body(Query._get_query(args, runner_manager.get_info()))
            Query._run_query(runner_manager, args, query_body)
        except ValueError as e:
            print(f'Unable to run tql Query. You must provide a query or query file. {e}')

    @staticmethod
    def run_flow(runner_manager: RunnerManager,
                 args: Namespace) -> None:
        """
        Runs a Query using the API Path, Auth and Query body.
        """
        query_body = Query._get_flow_request_body(Query._get_query(args, runner_manager.get_info()))
        Query._run_query(runner_manager, args, query_body)

    @staticmethod
    def _standardize_file(info: Info,
                          fn: str) -> str:
        """
        corrects filename short cuts like ~/
        uses the env set input path (if set)
        allows the user to override the env input path if they specify a path

        fixme - move to a utility like class higher up
        """
        if fn is None or fn == '':
            raise FileExistsError(fn)

        # standardize home shortcut
        if fn.startswith('~/'):
            home = expanduser('~')
            fn = fn.replace('~/', f'{home}/')

        # just a filename or a path+filename?
        paths = os.path.split(fn)
        if len(paths) == 1:
            # just a filename, so...
            # if there is a valid input path specified, use it
            # else use the input path and the filename
            if info.get_input_path() is not None and os.path.exists(info.get_input_path()):
                fn = os.path.join(info.get_input_path(), fn)

        if not os.path.exists(fn):
            raise FileNotFoundError(fn)

        # return the standardized filename
        return fn

    @staticmethod
    def _get_tql_request_body(query: str) -> Dict:
        """
        Creates the Request body for the Query API Call.
        """
        return {
            'queryTql': query
        }

    @staticmethod
    def _get_flow_request_body(query: str) -> Dict:
        """
        Creates the Request body for the Query API Call.
        """
        return {
            'queryTqlFlow': query
        }

    @staticmethod
    def _get_api_path(info: Info,
                      args: Namespace) -> str:
        """
        Creates the API path for each type of Query based on Data Source.
        """
        if info.get_target() == Target.BURSTCHAIN:
            if info.get_asset_type() == 'assets' or info.get_asset_type() is None:
                path = f'{info.get_url(args)}/api/burstchain/{info.get_client(args)}/' \
                       f'{args.collection}/assets/query'
            else:
                path = f'{info.get_url(args)}/api/burstchain/{info.get_client(args)}/' \
                       f'{args.collection}/assets/interrogate'
        else:
            path = f'{info.get_url(args)}/api/iq/{args.collection}/query'

        return path

    @staticmethod
    def _get_auth(info: Info,
                  arg: Namespace,
                  runner_manager: RunnerManager) -> (Dict, HTTPBasicAuth):
        """
        Creates the Auth (Basic vs Private ID) based on Data Source

        return tuple will contain either:
          a header with authorization set (id ID)
          OR
          the http basic auth for requests
        """
        if info.get_target() == Target.BURSTCHAIN:
            return {
                       'Authorization': 'ID ' + info.get_private_id()
                   }, None
        else:
            return None, runner_manager.get_basic_auth(info, arg)

    @staticmethod
    def _do_output(runner_manager: RunnerManager,
                   args: Namespace,
                   resp: requests):
        if args.output:
            Query._save_response(resp, runner_manager.get_info(), args)
        else:
            Query._print_response(resp, runner_manager.get_info())

    @staticmethod
    def _print_response(resp: requests,
                        info: Info) -> None:
        """
        Prints a General response for Queries
        """
        # print response
        if resp.status_code == 200:
            if info.get_target():
                print(f'   records:')
                print(json.dumps(resp.json().get('records'), indent=2))
            else:
                print(f'   assets:')
                print(json.dumps(resp.json().get('assets'), indent=2))

            print()
        else:
            print(f'ERROR RESPONSE')
            print(f'   error:     {resp.json().get("error")}')
            print()

    @staticmethod
    def _save_response(resp: requests,
                       info: Info,
                       args: Namespace):
        """
        Saves a the results from the Query
        """
        if resp.status_code == 200:
            out = info.get_output_path() + args.output
            print(f'   output file:  {out}')
            with open(out, 'wt') as f:
                f.write(json.dumps(resp.json().get('records'), indent=2))

            print()
        else:
            print(f'ERROR RESPONSE')
            print(f'   error:     {resp.json().get("error")}')
            print()

    @staticmethod
    def _run_query(runner_manager: RunnerManager,
                   args: Namespace,
                   body: dict):
        """
        Runs a Query using the API Path, Auth and Query body.
        """
        try:
            if args.profile is not None:
                runner_manager.set_profile(args.profile)

            query_body = body
            api_path = Query._get_api_path(runner_manager.get_info(), args)
            auth = Query._get_auth(runner_manager.get_info(), args, runner_manager)

            if runner_manager.get_info().get_target() == Target.BURSTCHAIN:
                resp = ApiRunner.run_chain_post(api_path, query_body, auth[0])
            else:
                resp = ApiRunner.run_post(api_path, query_body, auth[1])

            Query._do_output(runner_manager, args, resp)

        except Exception as e:
            print(f'Unable to run query, {e}')

    @staticmethod
    def _get_query_from_file(filename: str,
                             info: Info):
        try:
            filename = Query._standardize_file(info, filename)
            filename = info.get_input_path() + filename

            with open(filename, 'r') as file:
                lines = file.read()
            return lines
        except Exception as e:
            # catch everything, to prevent the interpreter from being killed
            assert f'ERROR: {e}'

    @staticmethod
    def _get_query(args: Namespace,
                   info: Info):
        if args.query is None or args.query == '':
            if args.file is None or args.file == '':
                assert ValueError('No Query or File Query Found.')
            else:
                return Query._get_query_from_file(args.file, info)
        else:
            return args.query
